@@ -2,7 +2,7 @@
 
 # PYTHON_ARGCOMPLETE_OK
 
-import os, sys, re, json, requests, xmltodict, shutil
+import os, sys, re, json, requests, xmltodict, shutil, base64, time
 
 from copy import copy
 from datetime import datetime
@@ -19,7 +19,6 @@ squirrel = Squirrel()
 
 dts = '%Y-%m-%dT%H:%M:%S'
 
-
 #________________________________________________________________
 class Silencer(object):
 	def write(self, *args, **kwargs):
@@ -124,9 +123,9 @@ class STEP(object):
 		if self.verbose:
 			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
 		result = requests.put(url=url, auth=auth, headers=headers, params=params, data=body)
-		if result.status_code != 200 or self.verbose:
+		if result.status_code not in [200,201] or self.verbose:
 			sys.stderr.write('%s: %s\n'%(result, result.text))
-		return result
+		return result.text
 
 	
 	def post(self, path, body=None, params=None, headers=None):
@@ -145,7 +144,7 @@ class STEP(object):
 		result = requests.post(url=url, auth=auth, headers=headers, params=params, data=body)
 		if result.status_code not in [200,201] or self.verbose:
 			sys.stderr.write('%s: %s\n'%(result, result.text))
-		return result.json()
+		return result.text
 
 
 	def delete(self, path, body=None, params=None, headers=None):
@@ -164,7 +163,7 @@ class STEP(object):
 		result = requests.delete(url=url, auth=auth, headers=headers, params=params, data=body)
 		if result.status_code != 200 or self.verbose:
 			sys.stderr.write('%s: %s\n'%(result, result.text))
-		return result
+		return result.text
 
 	
 	def file(self, path, destination):
@@ -314,7 +313,6 @@ class Processes(STEP):
 		return super().get('background-processes/%s/attachments/%s/content'%(id,attachmentId))
 
 	
-
 #________________________________________________________________
 @args.command(name='instances')
 class Instances(STEP):
@@ -705,3 +703,206 @@ class Endpoints(STEP):
 		return super().put('%s/%s/invoke'%(self.base, id))
 											   
 
+#________________________________________________________________
+@args.command(name='imports')
+class Imports(STEP):
+	'''
+	MIME type imports
+	'''
+	
+	base = 'import'
+	
+	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
+		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
+		
+
+	@args.operation(name='import')
+	@args.parameter(name='id', help='importConfigurationId')
+	@args.parameter(name='process', short='p', help='process BGP description')
+	def importer(self, id, file, process=None):
+		'''
+		returns the process BGP ID for the import process
+		'''
+		params = {
+			"context" : self.context,
+			"workspace": self.workspace,
+		}
+		if process:
+			params['processDescription'] = process
+		with open(file,'rb') as input:
+			body = input.read()
+			headers = { 'Content-Type' : 'application/octet-stream' }
+			result = super().post('%s/%s'%(self.base, id), body=body, headers=headers, params=params)
+			return result.json()
+		
+
+#________________________________________________________________
+@args.command(name='exports')
+class Exports(STEP):
+	'''
+	MIME type exports
+	'''
+	
+	base = 'export'
+	
+	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
+		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
+		
+
+	@args.operation(name='export')
+	@args.parameter(name='id', help='exportConfigurationId')
+	@args.parameter(name='use_context', short='c', flag=True, help='useRequestContextWorkspace')
+	@args.parameter(name='process', short='p', help='process BGP description', default='triggered by rest')
+	@args.parameter(name='urls', short='u', nargs='*', help='STEP URLs')
+	def importer(self, id, use_context=None, process=None, urls=[]):
+		'''
+		returns the process BGP ID for the import process
+		'''
+		params = {
+			"context" : self.context,
+			"workspace": self.workspace,
+		}
+		if use_context:
+			params['useRequestContextWorkspace'] = json.dumps(use_context)
+		body = {
+			'stepUrls': urls,
+			'processDescription': process,
+		}
+		headers = { 'Content-Type' : 'application/json' }
+		result = super().post('%s/%s'%(self.base, id), body=json.dumps(body), headers=headers, params=params)
+		return result.json()
+
+
+#________________________________________________________________
+node_types = {
+	'P': 'product',
+	'E': 'entity',
+	'C': 'classification',
+	'A' : 'asset',
+	'a' : 'attribute',
+}
+
+#________________________________________________________________
+@args.command(name='workflow')
+class Workflow(STEP):
+
+	base = 'workflows'
+
+	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
+		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
+		
+
+	@args.operation(help='get a list of workflows')
+	def list(self):
+		return super().get('%s'%self.base)				
+
+	@args.operation(help='get workflow by id')
+	@args.parameter(name='id', help='the ID of workflow definition')
+	def get(self, id):
+		return super().get('%s/%s'%(self.base,id))
+
+	@args.operation(help='instantiate a workflow instance')
+	@args.parameter(name='workflow_id', help='the ID of workflow')
+	@args.parameter(name='node_id', help='the node ID')
+	@args.parameter(name='node_type', short=True, flag=True, oneof=node_types, help='core node type', default='P')
+	@args.parameter(name='message', short='m', help='process instance message')
+	@args.parameter(name='id_as_base64', short='b', flag=True, help='return ID raw, do not decode base64 json')
+	def start(self, workflow_id, node_id, node_type='P', message='', id_as_base64=False):
+		'''
+		instantiate a workflow process instance for node
+		'''
+		
+		body = json.dumps(dict(
+			node=dict(
+				id=node_id,
+				type=node_types[node_type]
+			),
+			message=message
+		))
+		
+		headers = { 'Content-Type' : 'application/json' }
+		result = super().post('%s/%s/instances'%(self.base, workflow_id), body=body, headers=headers)		
+
+		response = json.loads(result)
+		if not 'id' in response.keys():
+			return response
+				
+		if id_as_base64:
+			return response['id']
+			
+		instance = base64.b64decode(response['id']).decode('UTF-8')
+		return json.loads(instance)
+		
+	@args.operation(help='terminate a workflow instance')
+	@args.parameter(name='workflow_id', help='the ID of workflow')
+	@args.parameter(name='instance_id', help='the ID of instance')
+	def terminate(self, workflow_id, instance_id):
+		'''
+		terminate workflow process instance
+		'''
+		return super().delete('%s/%s/instances/%s'%(self.base, workflow_id, instance_id))		
+		
+
+#________________________________________________________________
+@args.command(name='tasks')
+class Task(STEP):
+
+	base = 'workflow-tasks'
+
+	def __init__(self, asXML=None, verbose=None, output=None, silent=True, hostname=None, username=None, context=None, workspace=None):
+		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent, hostname=hostname, username=username, context=context, workspace=workspace)
+		
+
+	@args.operation(help='get a list of workflows')
+	@args.parameter(name='workflow_id', help='the ID of workflow')
+	@args.parameter(name='state_id', short='s', help='state id', default='')
+	@args.parameter(name='node_id', help='the node ID')
+	@args.parameter(name='node_type', short=True, flag=True, oneof=node_types, help='core node type', default='P')
+	@args.parameter(name='message', short='m', help='process instance message')
+	@args.parameter(name='id_as_base64', short='b', flag=True, help='return ID raw, do not decode base64 json')
+	def search(self, workflow_id, state_id='', node_id=None, node_type='P', message='', id_as_base64=False):
+		'''
+		search for workflow instances
+		'''
+		
+		query = dict(
+			workflow=workflow_id,
+			state=state_id,
+		)
+		
+		if node_id:
+			query['node'] = dict(
+				id=node_id,
+				type=node_types[node_type]
+			)
+				
+		body = json.dumps(query)
+		
+		headers = { 
+			'Content-Type' : 'application/json',
+			'Accept' : 'application/json',
+		}
+			
+		result = super().post('%s/search'%(self.base), body=body, headers=headers)
+		
+		#print(result)
+		
+		items = json.loads(result)
+		
+		if id_as_base64:
+			return items
+			
+		instances=[]
+		
+		for item in items:
+			instance = base64.b64decode(item)		
+			instances.append(json.loads(instance.decode('UTF-8')))
+		
+		return instances
+		
+	@args.operation(help='get workflow task by id')
+	@args.parameter(name='id', help='the ID of workflow task')
+	def get(self, id):
+		return super().get('%s/%s'%(self.base,id))
+		
+
