@@ -4,6 +4,8 @@
 
 import sys, os, re
 
+if os.path.dirname(sys.argv[0]) == '.': sys.path.insert(0, '..')
+
 from Argumental.Argue import Argue
 from Baubles.Colours import Colours
 from Perdy.parser import printXML
@@ -214,7 +216,7 @@ class STEP2UML(object):
 				if lovl:
 					cid = getAttribute(lovl, 'ListOfValueID')
 					ctype = self.lovs[cid]
-			xmi.makeAttribute('@base', ctype, None, _attribute, array=False)
+			xmi.makeAttribute('@base', ctype, cid.lower(), _attribute, array=False)
 
 			spec_desc = getAttribute(attribute, 'ProductMode')
 			if spec_desc == 'Normal':
